require 'oauth2'
require 'ihome'
require 'ihome/configuration'
require 'json'
require 'httparty'

module Ihome
  class Client
    public
    def initialize(opts)
      if opts[:auth_code]
        get_token!(opts[:auth_code])
      else
        authenticate!(opts[:access_token], opts[:refresh_token])
      end
    end

    def get_token!(code)
      @token = Ihome.oauth2_client.auth_code.get_token(code, {
         redirect_uri: Ihome.configuration.redirect_uri,
      })
    end

    def authenticate!(access_token, refresh_token = nil)
      @token = OAuth2::AccessToken.new(Ihome.oauth2_client, access_token, refresh_token: refresh_token)
    end

    def refresh!
      @token = @token.refresh!
    end

    def access_token
      @token.token
    end

    def refresh_token
      @token.refresh_token
    end

    def evrythng_api_key
      # This will only be stored after exchanging an authcode.
      # To get a new evrythng API key, reauthenticate with an authentication code
      @token.params['evrythng_api_key']
    end
  end

  class EvrythngClient
    def initialize(opts)
      @evrythng_api_key = opts[:evrythng_api_key]
    end

    def default_headers
      {
        'authorization' => @evrythng_api_key,
        'content-type'  => 'application/json',
        'accept'        => 'application/json'
      }
    end

    def get_devices
      response = HTTParty.get("https://api.evrythng.com/thngs", {
        :headers => default_headers
      })
      response.body
    end

    def send_powered_command(thng_id, state, source, outlet_number = 1)
      body = {
        'type'=> '_ControlPowerState',
        'thng'=> thng_id,
        'custom_fields'=> {
          'outlet_number'=> outlet_number,
          'state'=> state
        },
        'context'=> {
          'source'=> source
        }
      }.to_json

      HTTParty.post("https://api.evrythng.com/actions/_ControlPowerState", {
        headers: default_headers,
        body: body
      })
    end

    def update_name(thng_id, name)
      body = {
        "name"=> name
      }.to_json

      HTTParty.put("https://api.evrythng.com/thngs/#{thng_id}", {
        headers: default_headers,
        body: body
      })
    end
  end

  protected

  def self.oauth2_client
    @oauth2_client ||= begin
      raise ConfigurationMissingError unless @configuration
      OAuth2::Client.new(@configuration.client_id, @configuration.client_secret, {
        site: @configuration.site,
        authorize_url: @configuration.authorize_url,
        token_url: @configuration.token_url,
      })
    end
  end
end